@@ -64,7 +64,7 @@ normalize_qmistatus (const gchar *status)
 typedef struct {
     MMBroadbandBearerNovatelLte *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     MMPort *data;
     GCancellable *cancellable;
     GSimpleAsyncResult *result;
@@ -138,7 +138,7 @@ poll_connection_ready (MMBaseModem *modem,
     }
 
     if (is_qmistatus_disconnected (result)) {
-        mm_bearer_report_connection_status (MM_BEARER (bearer), MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+        mm_base_bearer_report_connection_status (MM_BASE_BEARER (bearer), MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
         g_source_remove (bearer->priv->connection_poller);
         bearer->priv->connection_poller = 0;
     }
@@ -149,8 +149,8 @@ poll_connection (MMBroadbandBearerNovatelLte *bearer)
 {
     MMBaseModem *modem = NULL;
 
-    g_object_get (MM_BEARER (bearer),
-                  MM_BEARER_MODEM, &modem,
+    g_object_get (MM_BASE_BEARER (bearer),
+                  MM_BASE_BEARER_MODEM, &modem,
                   NULL);
     mm_base_modem_at_command (
         modem,
@@ -282,10 +282,10 @@ connect_3gpp_authenticate (DetailedConnectContext *ctx)
     MMBearerProperties *config;
     gchar *command, *apn, *user, *password;
 
-    config = mm_bearer_peek_config (MM_BEARER (ctx->self));
-    apn = mm_at_serial_port_quote_string (mm_bearer_properties_get_apn (config));
-    user = mm_at_serial_port_quote_string (mm_bearer_properties_get_user (config));
-    password = mm_at_serial_port_quote_string (mm_bearer_properties_get_password (config));
+    config = mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self));
+    apn = mm_port_serial_at_quote_string (mm_bearer_properties_get_apn (config));
+    user = mm_port_serial_at_quote_string (mm_bearer_properties_get_user (config));
+    password = mm_port_serial_at_quote_string (mm_bearer_properties_get_password (config));
     command = g_strdup_printf ("$NWQMICONNECT=,,,,,,%s,,,%s,%s",
                                apn, user, password);
     g_free (apn);
@@ -307,8 +307,8 @@ connect_3gpp_authenticate (DetailedConnectContext *ctx)
 static void
 connect_3gpp (MMBroadbandBearer *self,
               MMBroadbandModem *modem,
-              MMAtSerialPort *primary,
-              MMAtSerialPort *secondary,
+              MMPortSerialAt *primary,
+              MMPortSerialAt *secondary,
               GCancellable *cancellable,
               GAsyncReadyCallback callback,
               gpointer user_data)
@@ -347,7 +347,7 @@ connect_3gpp (MMBroadbandBearer *self,
 typedef struct {
     MMBroadbandBearer *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     MMPort *data;
     GSimpleAsyncResult *result;
     gint retries;
@@ -356,7 +356,7 @@ typedef struct {
 static DetailedDisconnectContext *
 detailed_disconnect_context_new (MMBroadbandBearer *self,
                                  MMBroadbandModem *modem,
-                                 MMAtSerialPort *primary,
+                                 MMPortSerialAt *primary,
                                  MMPort *data,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
@@ -486,8 +486,8 @@ disconnect_3gpp_check_status (MMBaseModem *modem,
 static void
 disconnect_3gpp (MMBroadbandBearer *self,
                  MMBroadbandModem *modem,
-                 MMAtSerialPort *primary,
-                 MMAtSerialPort *secondary,
+                 MMPortSerialAt *primary,
+                 MMPortSerialAt *secondary,
                  MMPort *data,
                  guint cid,
                  GAsyncReadyCallback callback,
@@ -517,7 +517,7 @@ disconnect_3gpp (MMBroadbandBearer *self,
 
 /*****************************************************************************/
 
-MMBearer *
+MMBaseBearer *
 mm_broadband_bearer_novatel_lte_new_finish (GAsyncResult *res,
                                             GError **error)
 {
@@ -532,9 +532,9 @@ mm_broadband_bearer_novatel_lte_new_finish (GAsyncResult *res,
         return NULL;
 
     /* Only export valid bearers */
-    mm_bearer_export (MM_BEARER (bearer));
+    mm_base_bearer_export (MM_BASE_BEARER (bearer));
 
-    return MM_BEARER (bearer);
+    return MM_BASE_BEARER (bearer);
 }
 
 void
@@ -550,15 +550,15 @@ mm_broadband_bearer_novatel_lte_new (MMBroadbandModemNovatelLte *modem,
         cancellable,
         callback,
         user_data,
-        MM_BEARER_MODEM, modem,
-        MM_BEARER_CONFIG, config,
+        MM_BASE_BEARER_MODEM, modem,
+        MM_BASE_BEARER_CONFIG, config,
         NULL);
 }
 
 static void
 mm_broadband_bearer_novatel_lte_init (MMBroadbandBearerNovatelLte *self)
 {
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                               MM_TYPE_BROADBAND_BEARER_NOVATEL_LTE,
                                               MMBroadbandBearerNovatelLtePrivate);
 
