@@ -182,7 +182,7 @@ ip_configuration_query_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        mbim_message_command_done_get_result (response, &error) &&
+        mm_mbim_helper_response_get_result (response, &error) &&
         mbim_message_ip_configuration_response_parse (
             response,
             NULL, /* sessionid */
@@ -459,7 +459,7 @@ connect_set_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        (mbim_message_command_done_get_result (response, &error) ||
+        (mm_mbim_helper_response_get_result (response, &error) ||
          error->code == MBIM_STATUS_ERROR_FAILURE)) {
         GError *inner_error = NULL;
 
@@ -518,7 +518,7 @@ provisioned_contexts_query_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        mbim_message_command_done_get_result (response, &error) &&
+        mm_mbim_helper_response_get_result (response, &error) &&
         mbim_message_provisioned_contexts_response_parse (
             response,
             &provisioned_contexts_count,
@@ -571,7 +571,7 @@ packet_service_set_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        (mbim_message_command_done_get_result (response, &error) ||
+        (mm_mbim_helper_response_get_result (response, &error) ||
          error->code == MBIM_STATUS_ERROR_FAILURE)) {
         GError *inner_error = NULL;
 
@@ -950,6 +950,15 @@ disconnect_finish (MMBaseBearer *self,
     return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
 }
 
+static void
+reset_bearer_connection (MMBearerMbim *self)
+{
+    if (self->priv->data) {
+        mm_port_set_connected (self->priv->data, FALSE);
+        g_clear_object (&self->priv->data);
+    }
+}
+
 static void disconnect_context_step (DisconnectContext *ctx);
 
 static void
@@ -968,7 +977,7 @@ disconnect_set_ready (MbimDevice *device,
         GError *inner_error = NULL;
         gboolean result = FALSE, parsed_result = FALSE;
 
-        result = mbim_message_command_done_get_result (response, &error);
+        result = mm_mbim_helper_response_get_result (response, &error);
         /* Parse the response only for the cases we need to */
         if (result ||
             g_error_matches (error, MBIM_STATUS_ERROR, MBIM_STATUS_ERROR_FAILURE) ||
@@ -1064,8 +1073,7 @@ disconnect_context_step (DisconnectContext *ctx)
 
     case DISCONNECT_STEP_LAST:
         /* Port is disconnected; update the state */
-        mm_port_set_connected (ctx->self->priv->data, FALSE);
-        g_clear_object (&ctx->self->priv->data);
+        reset_bearer_connection (ctx->self);
 
         g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
         disconnect_context_complete_and_free (ctx);
@@ -1126,6 +1134,20 @@ mm_bearer_mbim_get_session_id (MMBearerMbim *self)
 
 /*****************************************************************************/
 
+static void
+report_connection_status (MMBaseBearer *self,
+                          MMBearerConnectionStatus status)
+{
+    if (status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED)
+        /* Cleanup all connection related data */
+        reset_bearer_connection (MM_BEARER_MBIM (self));
+
+    /* Chain up parent's report_connection_status() */
+    MM_BASE_BEARER_CLASS (mm_bearer_mbim_parent_class)->report_connection_status (self, status);
+}
+
+/*****************************************************************************/
+
 MMBaseBearer *
 mm_bearer_mbim_new (MMBroadbandModemMbim *modem,
                     MMBearerProperties *config,
@@ -1221,6 +1243,7 @@ mm_bearer_mbim_class_init (MMBearerMbimClass *klass)
     base_bearer_class->connect_finish = connect_finish;
     base_bearer_class->disconnect = disconnect;
     base_bearer_class->disconnect_finish = disconnect_finish;
+    base_bearer_class->report_connection_status = report_connection_status;
 
     properties[PROP_SESSION_ID] =
         g_param_spec_uint (MM_BEARER_MBIM_SESSION_ID,
