@@ -183,6 +183,10 @@ mm_mobile_equipment_error_from_mbim_nw_error (MbimNwError nw_error)
         return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
                             MM_MOBILE_EQUIPMENT_ERROR_GPRS_IMSI_UNKNOWN_IN_HLR,
                             "IMSI unknown in HLR");
+    case MBIM_NW_ERROR_ILLEGAL_MS:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_ILLEGAL_MS,
+                            "Illegal MS");
     case MBIM_NW_ERROR_IMSI_UNKNOWN_IN_VLR:
         return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
                             MM_MOBILE_EQUIPMENT_ERROR_GPRS_IMSI_UNKNOWN_IN_VLR,
@@ -227,6 +231,30 @@ mm_mobile_equipment_error_from_mbim_nw_error (MbimNwError nw_error)
         return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
                             MM_MOBILE_EQUIPMENT_ERROR_GPRS_CONGESTION,
                             "Congestion");
+    case MBIM_NW_ERROR_GSM_AUTHENTICATION_UNACCEPTABLE:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_USER_AUTHENTICATION_FAILED,
+                            "GSM authentication unacceptable");
+    case MBIM_NW_ERROR_NOT_AUTHORIZED_FOR_CSG:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_USER_AUTHENTICATION_FAILED,
+                            "Not authorized for this CSG");
+    case MBIM_NW_ERROR_MISSING_OR_UNKNOWN_APN:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_MISSING_OR_UNKNOWN_APN,
+                            "Missing or unknown APN");
+    case MBIM_NW_ERROR_SERVICE_OPTION_NOT_SUPPORTED:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_SERVICE_OPTION_NOT_SUPPORTED,
+                            "Service option not supported");
+    case MBIM_NW_ERROR_REQUESTED_SERVICE_OPTION_NOT_SUBSCRIBED:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_SERVICE_OPTION_NOT_SUBSCRIBED,
+                            "Requested service option not subscribed");
+    case MBIM_NW_ERROR_SERVICE_OPTION_TEMPORARILY_OUT_OF_ORDER:
+        return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                            MM_MOBILE_EQUIPMENT_ERROR_GPRS_SERVICE_OPTION_OUT_OF_ORDER,
+                            "Service option temporarily out of order");
     default:
         return g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
                             MM_MOBILE_EQUIPMENT_ERROR_GPRS_UNKNOWN,
@@ -253,3 +281,35 @@ mm_sms_state_from_mbim_message_status (MbimSmsStatus status)
 
     return MM_SMS_STATE_UNKNOWN;
 }
+
+/*****************************************************************************/
+
+gboolean
+mm_mbim_helper_response_get_result (const MbimMessage  *message,
+                                    GError            **error)
+{
+#if MBIM_CHECK_VERSION (1,11,1)
+    return mbim_message_response_get_result (message, MBIM_MESSAGE_TYPE_COMMAND_DONE, error);
+#else
+    MbimMessageType type;
+
+    g_return_val_if_fail (message != NULL, FALSE);
+
+    type = mbim_message_get_message_type (message);
+
+    if (type == MBIM_MESSAGE_TYPE_FUNCTION_ERROR) {
+        if (error)
+            *error = mbim_message_error_get_error (message);
+        return FALSE;
+    }
+
+    if (type == MBIM_MESSAGE_TYPE_COMMAND_DONE)
+        return mbim_message_command_done_get_result (message, error);
+
+    g_set_error (error,
+                 MM_CORE_ERROR,
+                 MM_CORE_ERROR_INVALID_ARGS,
+                 "Unexpected response message type: 0x%04X", (guint32) type);
+    return FALSE;
+#endif
+}
