@@ -39,9 +39,18 @@ static void initable_iface_init (GInitableIface *iface);
 
 G_DEFINE_TYPE_EXTENDED (MMPluginManager, mm_plugin_manager, G_TYPE_OBJECT, 0,
                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
-                                               initable_iface_init));
+                                               initable_iface_init))
+
+enum {
+    PROP_0,
+    PROP_PLUGIN_DIR,
+    LAST_PROP
+};
 
 struct _MMPluginManagerPrivate {
+    /* Path to look for plugins */
+    gchar *plugin_dir;
+
     /* This list contains all plugins except for the generic one, order is not
      * important. It is loaded once when the program starts, and the list is NOT
      * expected to change after that.*/
@@ -50,6 +59,28 @@ struct _MMPluginManagerPrivate {
     MMPlugin *generic;
 };
 
+/*****************************************************************************/
+/* Look for plugin */
+
+MMPlugin *
+mm_plugin_manager_peek_plugin (MMPluginManager *self,
+                               const gchar *plugin_name)
+{
+    GList *l;
+
+    if (self->priv->generic && g_str_equal (plugin_name, mm_plugin_get_name (self->priv->generic)))
+        return self->priv->generic;
+
+    for (l = self->priv->plugins; l; l = g_list_next (l)) {
+        MMPlugin *plugin = MM_PLUGIN (l->data);
+
+        if (g_str_equal (plugin_name, mm_plugin_get_name (plugin)))
+            return plugin;
+    }
+
+    return NULL;
+}
+
 /*****************************************************************************/
 /* Find device support */
 
@@ -299,6 +330,84 @@ deferred_support_check_idle (PortProbeContext *port_probe_ctx)
     return FALSE;
 }
 
+static void
+suggest_single_port_probe_result (PortProbeContext *target_port_probe_ctx,
+                                  MMPlugin *suggested_plugin,
+                                  gboolean reschedule_deferred)
+{
+    gboolean forbidden_icera;
+
+    /* Plugin suggestions serve two different purposes here:
+     *  1) Finish all the probes which were deferred until suggested.
+     *  2) Suggest to other probes which plugin to test next.
+     *
+     * The exception here is when we suggest the GENERIC plugin.
+     * In this case, only purpose (1) is applied, this is, only
+     * the deferred until suggested probes get finished.
+     */
+
+    if (target_port_probe_ctx->best_plugin || target_port_probe_ctx->suggested_plugin)
+        return;
+
+    /* Complete tasks which were deferred until suggested */
+    if (target_port_probe_ctx->defer_until_suggested) {
+        /* Reset the defer until suggested flag; we consider this
+         * cancelled probe completed now. */
+        target_port_probe_ctx->defer_until_suggested = FALSE;
+
+        if (suggested_plugin) {
+            mm_dbg ("(Plugin Manager) (%s) [%s] deferred task completed, got suggested plugin",
+                    mm_plugin_get_name (suggested_plugin),
+                    g_udev_device_get_name (target_port_probe_ctx->port));
+            /* Advance to the suggested plugin and re-check support there */
+            target_port_probe_ctx->suggested_plugin = g_object_ref (suggested_plugin);
+            target_port_probe_ctx->current = g_list_find (target_port_probe_ctx->current,
+                                                          target_port_probe_ctx->suggested_plugin);
+        } else {
+            mm_dbg ("(Plugin Manager) [%s] deferred task cancelled, no suggested plugin",
+                    g_udev_device_get_name (target_port_probe_ctx->port));
+            target_port_probe_ctx->best_plugin = NULL;
+            target_port_probe_ctx->current = NULL;
+        }
+
+        /* Schedule checking support, which will end the operation */
+        if (reschedule_deferred) {
+            g_assert (target_port_probe_ctx->defer_id == 0);
+            target_port_probe_ctx->defer_id = g_idle_add ((GSourceFunc)deferred_support_check_idle,
+                                                          target_port_probe_ctx);
+        }
+        return;
+    }
+
+    /* If no plugin being suggested, done */
+    if (!suggested_plugin)
+        return;
+
+    /* The GENERIC plugin is NEVER suggested to others */
+    if (g_str_equal (mm_plugin_get_name (suggested_plugin), MM_PLUGIN_GENERIC_NAME))
+        return;
+
+    /* If the plugin has MM_PLUGIN_FORBIDDEN_ICERA set, we do *not* suggest
+     * the plugin to others. Icera devices may not reply to the icera probing
+     * in all ports, so if other ports need to be tested for icera support,
+     * they should all go on. */
+    g_object_get (suggested_plugin,
+                  MM_PLUGIN_FORBIDDEN_ICERA, &forbidden_icera,
+                  NULL);
+    if (forbidden_icera)
+        return;
+
+    /* We should *not* cancel probing in the port if the plugin being
+     * checked right now is not the one being suggested. Each port
+     * should run its probing independently, and we'll later decide
+     * which result applies to the whole device.
+     */
+    mm_dbg ("(Plugin Manager) (%s) [%s] suggested plugin for port",
+            mm_plugin_get_name (suggested_plugin),
+            g_udev_device_get_name (target_port_probe_ctx->port));
+    target_port_probe_ctx->suggested_plugin = g_object_ref (suggested_plugin);
+}
+
 static void
 suggest_port_probe_result (FindDeviceSupportContext *ctx,
                            PortProbeContext *origin,
@@ -309,72 +418,8 @@ suggest_port_probe_result (FindDeviceSupportContext *ctx,
     for (l = ctx->running_probes; l; l = g_list_next (l)) {
         PortProbeContext *port_probe_ctx = l->data;
 
-        /* Plugin suggestions serve two different purposes here:
-         *  1) Finish all the probes which were deferred until suggested.
-         *  2) Suggest to other probes which plugin to test next.
-         *
-         * The exception here is when we suggest the GENERIC plugin.
-         * In this case, only purpose (1) is applied, this is, only
-         * the deferred until suggested probes get finished.
-         */
-
-        if (port_probe_ctx != origin &&
-            !port_probe_ctx->best_plugin &&
-            !port_probe_ctx->suggested_plugin) {
-            /* If we got a task deferred until a suggestion comes,
-             * complete it */
-            if (port_probe_ctx->defer_until_suggested) {
-                /* Reset the defer until suggested flag; we consider this
-                 * cancelled probe completed now. */
-                port_probe_ctx->defer_until_suggested = FALSE;
-
-                if (suggested_plugin) {
-                    mm_dbg ("(Plugin Manager) (%s) [%s] deferred task completed, got suggested plugin",
-                            mm_plugin_get_name (suggested_plugin),
-                            g_udev_device_get_name (port_probe_ctx->port));
-                    /* Advance to the suggested plugin and re-check support there */
-                    port_probe_ctx->suggested_plugin = g_object_ref (suggested_plugin);
-                    port_probe_ctx->current = g_list_find (port_probe_ctx->current,
-                                                           port_probe_ctx->suggested_plugin);
-                } else {
-                    mm_dbg ("(Plugin Manager) [%s] deferred task cancelled, no suggested plugin",
-                            g_udev_device_get_name (port_probe_ctx->port));
-                    port_probe_ctx->best_plugin = NULL;
-                    port_probe_ctx->current = NULL;
-                }
-
-                /* Schedule checking support, which will end the operation */
-                g_assert (port_probe_ctx->defer_id == 0);
-                port_probe_ctx->defer_id = g_idle_add ((GSourceFunc)deferred_support_check_idle,
-                                                       port_probe_ctx);
-            }
-            /* We should *not* cancel probing in the port if the plugin being
-             * checked right now is not the one being suggested. Each port
-             * should run its probing independently, and we'll later decide
-             * which result applies to the whole device.
-             */
-            else if (suggested_plugin &&
-                     /* The GENERIC plugin is NEVER suggested to others */
-                     !g_str_equal (mm_plugin_get_name (suggested_plugin),
-                                   MM_PLUGIN_GENERIC_NAME)) {
-                gboolean forbidden_icera;
-
-                /* If the plugin has MM_PLUGIN_FORBIDDEN_ICERA set, we do *not* suggest
-                 * the plugin to others. Icera devices may not reply to the icera probing
-                 * in all ports, so if other ports need to be tested for icera support,
-                 * they should all go on. */
-                g_object_get (suggested_plugin,
-                              MM_PLUGIN_FORBIDDEN_ICERA, &forbidden_icera,
-                              NULL);
-
-                if (!forbidden_icera) {
-                    mm_dbg ("(Plugin Manager) (%s) [%s] suggested plugin for port",
-                            mm_plugin_get_name (suggested_plugin),
-                            g_udev_device_get_name (port_probe_ctx->port));
-                    port_probe_ctx->suggested_plugin = g_object_ref (suggested_plugin);
-                }
-            }
-        }
+        if (port_probe_ctx != origin)
+            suggest_single_port_probe_result (port_probe_ctx, suggested_plugin, TRUE);
     }
 }
 
@@ -477,6 +522,25 @@ plugin_supports_port_ready (MMPlugin *plugin,
 
 
     case MM_PLUGIN_SUPPORTS_PORT_DEFER_UNTIL_SUGGESTED:
+        /* If we're deferred until suggested, but there is already a plugin
+         * suggested in the parent device context, grab it. This may happen if
+         * e.g. a wwan interface arrives *after* a port has already been probed.
+         */
+        if (!port_probe_ctx->suggested_plugin) {
+            MMPlugin *device_plugin;
+
+            /* Get info about the currently scheduled plugin in the device */
+            device_plugin = (MMPlugin *)mm_device_peek_plugin (port_probe_ctx->parent_ctx->device);
+            if (device_plugin) {
+                mm_dbg ("(Plugin Manager) (%s) [%s] task deferred until result suggested and got suggested plugin",
+                        mm_plugin_get_name (device_plugin),
+                        g_udev_device_get_name (port_probe_ctx->port));
+                /* Flag it as deferred before suggesting probe result */
+                port_probe_ctx->defer_until_suggested = TRUE;
+                suggest_single_port_probe_result (port_probe_ctx, device_plugin, FALSE);
+            }
+        }
+
         /* If we arrived here and we already have a plugin suggested, use it */
         if (port_probe_ctx->suggested_plugin) {
             if (port_probe_ctx->suggested_plugin == plugin) {
@@ -774,19 +838,19 @@ load_plugins (MMPluginManager *self,
     const gchar *fname;
     gchar *plugindir_display = NULL;
 
-	if (!g_module_supported ()) {
+    if (!g_module_supported ()) {
         g_set_error (error,
                      MM_CORE_ERROR,
                      MM_CORE_ERROR_UNSUPPORTED,
                      "GModules are not supported on your platform!");
         goto out;
-	}
+    }
 
     /* Get printable UTF-8 string of the path */
-    plugindir_display = g_filename_display_name (PLUGINDIR);
+    plugindir_display = g_filename_display_name (self->priv->plugin_dir);
 
     mm_dbg ("Looking for plugins in '%s'", plugindir_display);
-    dir = g_dir_open (PLUGINDIR, 0, NULL);
+    dir = g_dir_open (self->priv->plugin_dir, 0, NULL);
     if (!dir) {
         g_set_error (error,
                      MM_CORE_ERROR,
@@ -803,7 +867,7 @@ load_plugins (MMPluginManager *self,
         if (!g_str_has_suffix (fname, G_MODULE_SUFFIX))
             continue;
 
-        path = g_module_build_path (PLUGINDIR, fname);
+        path = g_module_build_path (self->priv->plugin_dir, fname);
         plugin = load_plugin (path);
         g_free (path);
 
@@ -847,11 +911,13 @@ out:
 }
 
 MMPluginManager *
-mm_plugin_manager_new (GError **error)
+mm_plugin_manager_new (const gchar *plugin_dir,
+                       GError **error)
 {
     return g_initable_new (MM_TYPE_PLUGIN_MANAGER,
                            NULL,
                            error,
+                           MM_PLUGIN_MANAGER_PLUGIN_DIR, plugin_dir,
                            NULL);
 }
 
@@ -864,6 +930,43 @@ mm_plugin_manager_init (MMPluginManager *manager)
                                                  MMPluginManagerPrivate);
 }
 
+static void
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
+{
+    MMPluginManagerPrivate *priv = MM_PLUGIN_MANAGER (object)->priv;
+
+    switch (prop_id) {
+    case PROP_PLUGIN_DIR:
+        g_free (priv->plugin_dir);
+        priv->plugin_dir = g_value_dup_string (value);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+static void
+get_property (GObject *object,
+              guint prop_id,
+              GValue *value,
+              GParamSpec *pspec)
+{
+    MMPluginManagerPrivate *priv = MM_PLUGIN_MANAGER (object)->priv;
+
+    switch (prop_id) {
+    case PROP_PLUGIN_DIR:
+        g_value_set_string (value, priv->plugin_dir);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
 static gboolean
 initable_init (GInitable *initable,
                GCancellable *cancellable,
@@ -885,13 +988,16 @@ dispose (GObject *object)
     }
     g_clear_object (&self->priv->generic);
 
+    g_free (self->priv->plugin_dir);
+    self->priv->plugin_dir = NULL;
+
     G_OBJECT_CLASS (mm_plugin_manager_parent_class)->dispose (object);
 }
 
 static void
 initable_iface_init (GInitableIface *iface)
 {
-	iface->init = initable_init;
+    iface->init = initable_init;
 }
 
 static void
@@ -903,4 +1009,15 @@ mm_plugin_manager_class_init (MMPluginManagerClass *manager_class)
 
     /* Virtual methods */
     object_class->dispose = dispose;
+    object_class->set_property = set_property;
+    object_class->get_property = get_property;
+
+    /* Properties */
+    g_object_class_install_property
+        (object_class, PROP_PLUGIN_DIR,
+         g_param_spec_string (MM_PLUGIN_MANAGER_PLUGIN_DIR,
+                              "Plugin directory",
+                              "Where to look for plugins",
+                              NULL,
+                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
